package org.eclipse.jwt.transformations.base.handlers;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xmi.util.XMLProcessor;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jwt.transformations.base.TransformationException;
import org.eclipse.jwt.transformations.base.TransformationRegistry;
import org.eclipse.jwt.we.editors.actions.WEActionHandler;
import org.eclipse.jwt.we.misc.util.GeneralHelper;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.ui.IWorkbenchPart;



/**
 * does only export for now
 * @author mistria
 *
 */
public class ExportTransformationHandler extends WEActionHandler {
	private String inFilePath = null;
	private String outFilePath = null;
	private Button checkBox = null;
	private Combo transformationsCombo;
	
	
	public ExportTransformationHandler() {
		// requires open WEEditor
		super(true);
		setEnabled(true);
	}

	/**
	 * Refresh enabled state if selection has changed.
	 */
	@Override
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.commands.AbstractHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		// Creation de l'ihm
		final Display display = Display.getCurrent();
		final Shell shell = new Shell(display);
		shell.setText("Transform JWT into...");
		GridLayout shellLayout = new GridLayout();
		shellLayout.numColumns = 1;
		shell.setLayout(shellLayout);
		Composite group0 = new Composite(shell, SWT.None);
		group0.setLayout(new RowLayout(SWT.HORIZONTAL));
		final Label compositeLabel = new Label(group0, SWT.LEFT);
		compositeLabel.setText("Please choose the input and output files of the JWT to XPDL transformation: ");
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		GridData inFilegridData = new GridData();
		inFilegridData.widthHint = 530;
		inFilegridData.horizontalIndent = 24;
		inFilegridData.verticalAlignment = GridData.CENTER;
		inFilegridData.horizontalAlignment = GridData.FILL;
		GridData outFilegridData = new GridData();
		outFilegridData.widthHint = 530;
		outFilegridData.verticalAlignment = GridData.CENTER;
		outFilegridData.horizontalAlignment = GridData.FILL;
		final Composite checkBoxGroup = new Composite(shell, SWT.None);
		checkBoxGroup.setLayout(gridLayout);
		Label checkBoxLabel = new Label(checkBoxGroup, SWT.None);
		checkBoxLabel.setText("Use external JWT Model: ");
		checkBox = new Button(checkBoxGroup, SWT.CHECK);
		final Composite inFileGroup = new Composite(shell, SWT.None);
		inFileGroup.setLayout(gridLayout);
		Label inFileLabel = new Label(inFileGroup, SWT.None);
		inFileLabel.setText("Input JWT file: ");
		final Text inFileText = new Text(inFileGroup, SWT.BORDER);
		inFileText.setText("");
		inFileText.setBounds(700, 700, 700, 700);
		inFileText.setLayoutData(inFilegridData);
		// creation d'un bouton
		Button loadInFileButton = new Button(inFileGroup, SWT.NONE);
		loadInFileButton.setText("Browse");
		loadInFileButton.setSize(100, 25);
		final Composite outFileGroup = new Composite(shell, SWT.None);
		GridData gd = new GridData();
		gd.heightHint = 0;
		inFileGroup.setLayoutData(gd);
		outFileGroup.setLayout(gridLayout);
		Label outFileLabel = new Label(outFileGroup, SWT.NONE);
		outFileLabel.setText("Output file: ");
		final Text outFileText = new Text(outFileGroup, SWT.BORDER);
		outFileText.setText("");
		outFileText.setBounds(700, 700, 700, 700);
		outFileText.setLayoutData(outFilegridData);
		// creation d'un bouton
		Button loadOutFileButton = new Button(outFileGroup, SWT.NONE);
		loadOutFileButton.setText("Browse");
		loadOutFileButton.setSize(100, 25);
		
		// Combo for transformations
		Composite transformChooser = new Composite(shell, SWT.None);
		transformChooser.setLayout(new RowLayout(SWT.None));
		final Label chooseTransformationText = new Label(transformChooser, SWT.None);
		chooseTransformationText.setText("Choose the transformation you want to apply");
		transformationsCombo = new Combo(transformChooser, SWT.READ_ONLY);
		Set<String> transformationsName = TransformationRegistry.getInstance().getTransformationsName();
		for (String trans : transformationsName) {
			transformationsCombo.add(trans);
		}
		final Label descriptionLabel = new Label(shell, SWT.BOLD);
		descriptionLabel.setText("Description: (no transformation selected)");
		
		Composite buttonGroup = new Composite(shell, SWT.NONE);
		GridLayout buttonsLayout = new GridLayout();
		buttonsLayout.numColumns = 3;
		// buttonsLayout.makeColumnsEqualWidth = true;
		buttonGroup.setLayout(buttonsLayout);
		GridData okGrid = new GridData();
		okGrid.horizontalAlignment = GridData.FILL;
		GridData buttonsGridData = new GridData();
		buttonsGridData.horizontalAlignment = GridData.END;
		buttonsGridData.verticalAlignment = GridData.CENTER;
		buttonGroup.setLayoutData(buttonsGridData);
		final Button ok = new Button(buttonGroup, SWT.PUSH);
		ok.setEnabled(false);
		Button cancel = new Button(buttonGroup, SWT.PUSH);
		cancel.setText("Cancel");
		ok.setText("Ok");
		ok.setLayoutData(okGrid);
		ok.setEnabled(false);
		cancel.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				shell.dispose();
			}
		});
		loadInFileButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				// ouverture d'une fenetre de selection de fichiers
				FileDialog dialog = new FileDialog(shell, SWT.OPEN);
				dialog.setFilterExtensions(new String[] { "*.workflow", "*.agilpro", "*.*" });
				inFilePath = dialog.open();
				if ((inFilePath != null) && (inFilePath.length() != 0)) {
					inFileText.setText(inFilePath);
					ok.setEnabled(hasAllRequiredInfo());
				}
			}
		});
		loadOutFileButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				// ouverture d'une fenetre de selection de fichiers
				FileDialog dialog = new FileDialog(shell, SWT.OPEN);
				dialog.setFilterExtensions(new String[] { "*.xpdl", "*.*" });
				outFileText.setText(dialog.open());
				if ((outFilePath != null) && (outFilePath.length() != 0)) {
					outFileText.setText(outFilePath);
					ok.setEnabled(hasAllRequiredInfo());
				}
			}
		});
		
		
		inFileText.addListener(SWT.Modify,  new Listener() {
			public void handleEvent(Event e) {
				inFilePath = inFileText.getText();
				ok.setEnabled(hasAllRequiredInfo());
			}
		});
		outFileText.addListener(SWT.Modify, new Listener() {
			public void handleEvent(Event e) {
				outFilePath = outFileText.getText();
				ok.setEnabled(hasAllRequiredInfo());
			}
		});

		checkBox.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				if (checkBox.getSelection()) {
					inFileGroup.setLayoutData(new GridData());
					inFileGroup.setVisible(true);
					shell.pack();
				} else {
					inFileGroup.setVisible(false);
					GridData gd = new GridData();
					gd.heightHint = 0;
					inFileGroup.setLayoutData(gd);
					shell.pack();
				}
				ok.setEnabled(hasAllRequiredInfo());
			}
		});
		transformationsCombo.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				ok.setEnabled(hasAllRequiredInfo());
				descriptionLabel.setText("Description: " + TransformationRegistry.getInstance().getTransformation(transformationsCombo.getText()).getDescription());
			}
		});
		ok.addListener(SWT.Selection, new Listener() {
			@SuppressWarnings("unchecked")
			public void handleEvent(Event e) {
				try {
					File outFile = new File(outFilePath);
					if (!outFile.exists()) {
						outFile.createNewFile();
					}
					if (!checkBox.getSelection()) {
						Map options = new HashMap();
						options.put(XMLResource.OPTION_ENCODING, "UTF-8");
						XMLProcessor proc = new XMLProcessor();
						File tempFile = null;
						FileOutputStream stream = null;
						try {
							tempFile = File.createTempFile("jwt", null);
							stream = new FileOutputStream(tempFile);
							Resource resource = GeneralHelper.getActiveInstance().getCurrentActivitySheet().getActivityModel().eResource();
							// TODO rather with eResource.save(), but keep the
							// old code as commented
							proc.save(stream, resource, options);
							stream.close();
						} catch (IOException e2) {
							e2.printStackTrace();
						}
						inFilePath = tempFile.getAbsolutePath();
					}
					if (!transformationsCombo.getText().equals(""))
						TransformationRegistry.getInstance().getTransformation(transformationsCombo.getText()).transform(inFilePath, outFilePath);
					
					ErrorDialog.openError(shell, "Success", "Succeeded transforming JWT model " + inFilePath + " to XPDL format "
							+ outFilePath, Status.OK_STATUS);
				} catch (IOException e1) {
					ErrorDialog.openError(shell, "JWT to XPDL error", "Can't start transforming JWT to XPDL : " + e1.getMessage(),
							Status.OK_STATUS);
				} catch (TransformationException e1) {
					ErrorDialog.openError(shell, "JWT to XPDL error", "Can't start transforming JWT to XPDL : " + e1.getMessage(),
							Status.OK_STATUS);
				}
				shell.dispose();
			}
		});
		shell.pack();
		centerOnScreen(display, shell);
		shell.open();
		return null;
	}

	// centrer une fenetre dans l'écran
	private void centerOnScreen(Display display, Shell shell) {
		Rectangle rect = display.getClientArea();
		Point size = shell.getSize();
		int x = (rect.width - size.x) / 2;
		int y = (rect.height - size.y) / 2;
		shell.setLocation(new Point(x, y));
	}

	private boolean hasAllRequiredInfo() {
		return (!checkBox.getSelection() || (inFilePath != null && inFilePath.length() != 0)) && outFilePath != null
				&& outFilePath.length() != 0 && transformationsCombo.getText().length() != 0;
	}
}
