package org.eclipse.jwt.transformations.base.init;

import java.io.IOException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jwt.transformations.base.TransformationException;
import org.eclipse.jwt.transformations.base.TransformationRegistry;
import org.eclipse.jwt.transformations.base.TransformationService;
import org.eclipse.ui.WorkbenchException;

public class ProcessServiceMembers {
	private static final String EXTENSION_POINT = "org.eclipse.jwt.compatibility.transformations";
	private static final String TRANSFORMATION_NAME_ATTRIBUTE = "name";
	private static final String CLASS_ATTRIBUTE = "class";
	private static final String DESCRIPTION_ATTRIBUTE = "description";

	public static void process(TransformationRegistry transformationRegistry) throws WorkbenchException {
		IExtensionRegistry registry = Platform.getExtensionRegistry();
		IExtensionPoint extensionPoint = registry
				.getExtensionPoint(EXTENSION_POINT);
		IConfigurationElement[] members = extensionPoint
				.getConfigurationElements();
		// For each service:
		for (int m = 0; m < members.length; m++) {
			IConfigurationElement member = members[m];
			String functionName = member
					.getAttribute(TRANSFORMATION_NAME_ATTRIBUTE);
			TransformationService proxy = new TransformationProxy(member);
			proxy.setDescription(member.getAttribute(DESCRIPTION_ATTRIBUTE));
			transformationRegistry.addTransformation(functionName, proxy);
		}
	}

	private static class TransformationProxy extends TransformationService {
		private TransformationService delegate = null; // The real callback.
		private IConfigurationElement element; // Function's configuration.
		private boolean invoked = false; // Called already.

		public TransformationProxy(IConfigurationElement element) {
			this.element = element;
		}

		@Override
		public final void transform(String inFilePath, String outFilePath) throws IOException, TransformationException {
			try {
				getDelegate();
			} catch (Exception ex) {
				throw new ArithmeticException("invalid function");
			}
			if (delegate == null) {
				throw new TransformationException("invalid function");
			}
			delegate.transform(inFilePath, outFilePath);
		}

		private final TransformationService getDelegate() throws Exception {
			if (invoked) {
				return delegate;
			}
			invoked = true;
			try {
				Object callback = element
						.createExecutableExtension(CLASS_ATTRIBUTE);
				if (!(callback instanceof TransformationService)) {
					throw new Exception("TransformationService creation error");
				}
				delegate = (TransformationService) callback;
			} catch (CoreException ex) {
				throw ex;
			}
			return delegate;
		}
	}
}
