<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
		xmlns:xmi="http://www.omg.org/XMI"
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		xmlns:core="org.eclipse.jwt/1.3.0/core"
		xmlns:organisations="org.eclipse.jwt/1.3.0/organisations"
		xmlns:processes="org.eclipse.jwt/1.3.0/processes"
		xmlns:jwt2xpdl="http://www.oracle.com/XSL/Transform/java/fr.openwide.scorware.jwt2xpdl.CustomXslFunctions">
    <xsl:output method="XML" encoding="UTF-8" indent="yes" version="1.0" omit-xml-declaration="no"/>
	<!-- 
		XPDL sample:
		http://www.wfmc.org/standards/docs/xpdl_sample/sample%20workflow%20process.xpdl
		XSD:
		http://wfmc.org/standards/docs/TC-1025_schema_10_xpdl.xsd
	 -->
    <xsl:template match="/core:Model">
    	<!-- $packageName : name of the first process -->
		<xsl:variable name="packageName" select="//elements[@xsi:type = 'processes:Activity'][1]/@name"/>

		<Package
				xmlns="http://www.wfmc.org/2002/XPDL1.0"
				xmlns:xpdl="http://www.wfmc.org/2002/XPDL1.0"
				xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
				xsi:schemaLocation="http://www.wfmc.org/2002/XPDL1.0 http://wfmc.org/standards/docs/TC-1025_schema_10_xpdl.xsd"
				Id="{$packageName}_Project"
				Name="{$packageName}">
			<PackageHeader>
				<XPDLVersion>1.0</XPDLVersion>
				<Vendor></Vendor><!-- TODO -->
				<Created></Created><!-- TODO -->
			</PackageHeader>
			
			<ConformanceClass GraphConformance="NON_BLOCKED"/>
			
			<WorkflowProcesses>
				<xsl:apply-templates select="elements[@xsi:type = 'processes:Activity']"/>
				<!-- same out for elements and subpackages/elements? -->
				<xsl:apply-templates select="subpackages/elements[@xsi:type = 'processes:Activity']"/>
			</WorkflowProcesses>
			
			<ExtendedAttributes>
				<ExtendedAttribute Name="MadeBy" Value="JWT2XPDL"/>
				<ExtendedAttribute Name="View" Value="Activity"/><!-- View value : "Activity" or "Participant" -->
			</ExtendedAttributes>
		</Package>			
    </xsl:template>


    <xsl:template match="elements[@xsi:type = 'processes:Activity']">
		<WorkflowProcess AccessLevel="PUBLIC" Name="{@name}" Id="{@name}">
			<ProcessHeader/>
			
			<RedefinableHeader>
				<Version>1.0</Version>
			</RedefinableHeader>
			
			<DataFields>
				<xsl:apply-templates select="//elements[@xsi:type = 'data:Data']"/>
			</DataFields>
			
			<Participants>
				<xsl:apply-templates select="//elements[@xsi:type = 'organisations:Role']"/>
			</Participants>
			
			<Activities>
				<xsl:apply-templates select="nodes"/>
			</Activities>
			
			<Transitions>
				<xsl:apply-templates select="edges"/>
			</Transitions>
		</WorkflowProcess>
    </xsl:template>


    <xsl:template match="nodes[@xsi:type = 'processes:InitialNode']">
    	<!-- nothing to do -->
    </xsl:template>
    
    
    <xsl:template match="nodes[@xsi:type = 'processes:Action']">
    	<xsl:call-template name="activity">
	    	<xsl:with-param name="type">node</xsl:with-param>
    	</xsl:call-template>
    </xsl:template>
    
    
    <xsl:template match="nodes[@xsi:type = 'processes:FinalNode']">
    	<xsl:call-template name="activity">
	    	<xsl:with-param name="type">final</xsl:with-param>
    	</xsl:call-template>
    </xsl:template>
    
    
    <xsl:template match="nodes[@xsi:type = 'processes:DecisionNode']">
    	<xsl:call-template name="activity">
	    	<xsl:with-param name="type">decision</xsl:with-param>
    	</xsl:call-template>
    </xsl:template>
    
    
    <xsl:template match="nodes[@xsi:type = 'processes:MergeNode']">
    	<xsl:call-template name="activity">
	    	<xsl:with-param name="type">merge</xsl:with-param>
    	</xsl:call-template>
    </xsl:template>
    
    
    <xsl:template name="activity">
    	<xsl:param name="type"/>
    	
		<Activity>
		    <xsl:choose>
		    	<xsl:when test="$type = 'final'">
					<!-- Proed wants Name="BonitaEnd" for final nodes -->
					<xsl:attribute name="Id">BonitaEnd</xsl:attribute>
					<xsl:attribute name="Name">BonitaEnd</xsl:attribute>
		    	</xsl:when>
		    	<xsl:otherwise>
					<xsl:attribute name="Id"><xsl:value-of select="@name"/></xsl:attribute>
					<xsl:attribute name="Name"><xsl:value-of select="@name"/></xsl:attribute>
		    	</xsl:otherwise>
		    </xsl:choose>
			
		    <xsl:choose>
		    	<xsl:when test="$type = 'decision' or $type = 'merge'">
					<Route/>
		    	</xsl:when>
		    	<xsl:otherwise>
		    		<!-- node, final -->
					<Implementation>
						<No/>
					</Implementation>
		    	</xsl:otherwise>
		    </xsl:choose>
			
			<xsl:if test="@performedBy">
				<Performer><xsl:value-of select="jwt2xpdl:getValueFromPath(@performedBy, /core:Model)"/></Performer>
			</xsl:if>
			
			<StartMode>
			    <xsl:choose>
			    	<xsl:when test="$type = 'node'">
						<Manual/><!-- TODO -->
			    	</xsl:when>
			    	<xsl:otherwise>
			    		<!-- decision, merge, final: always automatic -->
						<Automatic/>
			    	</xsl:otherwise>
			    </xsl:choose>
			</StartMode>
			
			<TransitionRestrictions>
				<TransitionRestriction>
					<Join>
						<xsl:attribute name="Type">XOR</xsl:attribute>
						<!-- TODO -->
						<!--
					    <xsl:choose>
					    	<xsl:when test="$type = 'final'">
								<xsl:attribute name="Type">XOR</xsl:attribute>
					    	</xsl:when>
					    	<xsl:otherwise>
								<xsl:attribute name="Type">AND</xsl:attribute>
					    	</xsl:otherwise>
					    </xsl:choose>
					    -->
					</Join>
				</TransitionRestriction>
			</TransitionRestrictions>
			
			<ExtendedAttributes>
				<ExtendedAttribute Name="XOffsetParticipantView" Value="0"/><!-- TODO -->
				<ExtendedAttribute Name="YOffsetParticipantView" Value="0"/><!-- TODO -->
				<ExtendedAttribute Name="XOffset" Value="{Location/@x}"/>
				<ExtendedAttribute Name="YOffset" Value="{Location/@y}"/>
				
			    <xsl:choose>
			    	<xsl:when test="Size/@height">
				    	<ExtendedAttribute Name="CellHeight" Value="Size/@height"/>
			    	</xsl:when>
			    	<xsl:otherwise>
				    	<ExtendedAttribute Name="CellHeight" Value="30"/>
			    	</xsl:otherwise>
			    </xsl:choose>
			    
			    <xsl:choose>
			    	<xsl:when test="Size/@width">
				    	<ExtendedAttribute Name="CellWidth" Value="Size/@width"/>
			    	</xsl:when>
			    	<xsl:otherwise>
				    	<ExtendedAttribute Name="CellWidth" Value="150"/>
			    	</xsl:otherwise>
			    </xsl:choose>
			    
				<!-- TODO Propagated properties -->
				<!--
				<ExtendedAttribute Name="property" Value="approvalResult">
					<Propagated>Yes</Propagated>
				</ExtendedAttribute>
				 -->
			</ExtendedAttributes>
		</Activity>
	</xsl:template>
	
	
    <xsl:template match="elements[@xsi:type = 'organisations:Role']">
	    <Participant Id="{@name}" Name="{@name}">
			<ParticipantType Type="HUMAN"/><!-- TODO -->
			<ExtendedAttributes>
				<ExtendedAttribute Name="NewParticipant" Value="true"/><!-- TODO -->
				<ExtendedAttribute Name="XOffset" Value="0"/><!-- TODO -->
				<ExtendedAttribute Name="YOffset" Value="0"/><!-- TODO -->
				<ExtendedAttribute Name="icon" Value="{@icon}"/>
			</ExtendedAttributes>
		</Participant>
    </xsl:template>

	
    <xsl:template match="edges">
	    <!--
	    	traduction of @source and @target:
	    	//@elements.0/@nodes.1 -> //elements[1]/nodes[2]/@name -> evaluate XPath
	    -->
	    <xsl:variable name="source" select="jwt2xpdl:getValueFromPath(@source, /core:Model)"/>

	    <xsl:if test="$source != ''">
		    <xsl:variable name="target" select="jwt2xpdl:getValueFromPath(@target, /core:Model)"/>
		    <xsl:variable name="id" select="translate(concat($source, '_', $target), ' ', '_')"/>
	    	
			<Transition Id="{$id}" Name="{$source}_{$target}" From="{$source}" To="{$target}">
				<!--
					JWT:
			        <guard name="Disprove" textualdescription="Disprove" shortdescription="isApproved.isApproved == false">
			          <detailedSpecification data="//@subpackages.2/@subpackages.1/@elements.4" attribute="isApproved" value="false"/>
			        </guard>
			        shortdescription="(Branche.Branche == tests &amp;&amp; isApproved.isApproved != true) || Reisemittel.Reisemittel == ok"
			        
			        XPDL:
			        <Condition Type="CONDITION">isApproved.equals("true")</Condition>
         			<Condition Type="CONDITION">(Branche.equals("tests") &amp;&amp; !isApproved.equals("true")) || Reisemittel.equals("ok")</Condition>
				-->
				<xsl:if test="guard">
					<!-- <xsl:variable name="data" select="jwt2xpdl:getValueFromPath(guard/detailedSpecification/@data, /core:Model)"/> -->
					<!--
					<xsl:variable name="attribute" select="guard/detailedSpecification/@attribute"/>
					<xsl:variable name="value" select="guard/detailedSpecification/@value"/>
					<Condition Type="CONDITION"><xsl:value-of select="concat($attribute, '.equals(&quot;', $value, '&quot;)')"/></Condition>
					-->
					<Condition Type="CONDITION"><xsl:value-of select="jwt2xpdl:getCondition(guard/@shortdescription, guard, /core:Model)"/></Condition>
				</xsl:if>
			</Transition>
		</xsl:if>
    </xsl:template>

	
    <xsl:template match="elements[@xsi:type = 'data:Data']">
		<DataField Id="{@name}" Name="{@name}">
			<DataType>
				<BasicType Type="STRING"/>
				<!-- TODO
					BasicType
						STRING
						FLOAT
						INTEGER
						REFERENCE
						DATETIME
						BOOLEAN
						PERFORMER
					DeclaredType
					SchemaType
					ExternalReference
					RecordType
					UnionType
					EnumerationType
					ArrayType
					ListType
				 -->
			</DataType>
		    <!-- TODO
				InitialValue
				Length
				Description
		    -->
			<ExtendedAttributes>
				<ExtendedAttribute Name="icon" Value="{@icon}"/>
			</ExtendedAttributes>
		</DataField>
    </xsl:template>

</xsl:stylesheet>
