/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id : $ 
 * -------------------------------------------------------------------------
 */

package org.eclipse.jwt.transformations.xpdl.impl;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.namespace.NamespaceContext;

/**
 * @version $Rev: $ $Date: 2008/02/05 13:53:23 $
 * @author Guillaume Decarnin - Open Wide
 */
public class CustomNamespaceContext implements NamespaceContext {

    private final Map<String, String> mapUri;

    private final Map<String, String> mapPrefix;

    /**
     * @param mapUri
     *            key = prefix ; value = namespaceURI
     */
    public CustomNamespaceContext(final Map<String, String> mapUri) {
        this.mapUri = mapUri;

        // reverted map
        this.mapPrefix = new HashMap<String, String>();
        for (final Entry<String, String> entry : mapUri.entrySet()) {
            this.mapPrefix.put(entry.getValue(), entry.getKey());
        }
    }

    /**
     * Namespace with one URI.
     * 
     * @param prefix
     * @param uri
     */
    public CustomNamespaceContext(final String prefix, final String uri) {
        this.mapUri = new HashMap<String, String>();
        this.mapUri.put(prefix, uri);

        // reverted map
        this.mapPrefix = new HashMap<String, String>();
        this.mapPrefix.put(uri, prefix);
    }

    /**
     * @param prefix
     */
    public final String getNamespaceURI(final String prefix) {
        return this.mapUri.get(prefix);
    }

    /**
     * @param namespaceURI
     */
    public final String getPrefix(final String namespaceURI) {
        return this.mapPrefix.get(namespaceURI);
    }

    /**
     * @param namespaceURI
     */
    public final Iterator<String> getPrefixes(final String namespaceURI) {
        return this.mapPrefix.values().iterator();
    }
}
