package org.eclipse.jwt.transformations.xpdl.impl;

import java.io.StringReader;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 * @author gdecarnin
 *
 */
public class CustomXslFunctions {

    private static final String beforeOp = "^([^=<> ]+) ?"; // all chars without operator and space
    private static final String afterOp = " *([^&| ]+)([|& ]*)(.*)$"; // an optional space and all chars until the end
    
    private static final Pattern patternVar       = Pattern.compile("^(.*)([^\\.]+)\\.\\2(.*)$"); // *isApproved.isApproved*
    private static final Pattern patternEqual     = Pattern.compile(beforeOp + "==" + afterOp);
    private static final Pattern patternDifferent = Pattern.compile(beforeOp + "!=" + afterOp);
    private static final Pattern patternInf       = Pattern.compile(beforeOp + "<=" + afterOp);
    private static final Pattern patternSup       = Pattern.compile(beforeOp + "=>" + afterOp);
    private static final Pattern patternAnd       = Pattern.compile(beforeOp + "&&" + afterOp);
    private static final Pattern patternOr        = Pattern.compile(beforeOp + "||" + afterOp);

    /**
     * "//@elements.0/@nodes.1" -> "//elements[1]/nodes[2]/@name"
     * 
     * @param nodePath
     * @param root
     * @return
     */
    public static String getValueFromPath(Node nodePath, Node root) {
        String value = "";
        
        try {
            String originalPath = nodePath.getTextContent().split(" ")[0]; // TODO gérer le split(" ")
//            System.out.println("originalPath:" + originalPath);
            
            /*
             * building xPathExpression :
             * 1. start with "//@elements.0/@nodes.1"
             * 2. remove all "@" : "//elements.0/nodes.1"
             * 3. replace "//" by "/core:Model/"
             * 4. remove ".", add "[ ]" and incremente indexes : "//elements[1]/nodes[2]"
             * 5. append "/@name" : "//elements[1]/nodes[2]/@name"
             */
            StringBuilder xPathExpression = new StringBuilder();
            
            String path = originalPath;
            path = path.replaceAll("@", "");
            path = path.replaceAll("//", "/core:Model/");
            
            Pattern pattern = Pattern.compile("([^\\.]*)\\.(\\d+)([^/]*)"); // (any char).(integer)(any char)/
            Matcher matcher = pattern.matcher(path);
            while (matcher.find()) {
                /* 
                 * exemple:
                 * path://@elements.0/@nodes.1
                 * 
                 * iteration 1:
                 * groupe0://@elements.0
     "org.eclipse.jwt/1.3.0/core"            * groupe1://@elements
                 * groupe2:0
                 * groupe3:
                 * 
                 * iteration 2:
                 * groupe0:/@nodes.1
                 * groupe1:/@nodes
                 * groupe2:1
                 * groupe3:
                 */
                //for (int i = 0; i <= matcher.groupCount(); i++) {
                //    System.out.println("groupe" + i + ":" + matcher.group(i));
                //}
                
                int index = Integer.parseInt(matcher.group(2));
                index++;
                
                xPathExpression.append(matcher.group(1));
                xPathExpression.append('[');
                xPathExpression.append(index);
                xPathExpression.append(']');
            }
            
            String xPathExpressionName = xPathExpression.toString() + "/@name";
            String xPathExpressionType = xPathExpression.toString() + "/@xsi:type";
            
//            System.out.println("xPathExpression:" + xPathExpression);
            
//            System.out.println(createStringFromNode(root));
//            System.out.println(createStringFromNodeList(elements));
            
            XPath xPath = XPathFactory.newInstance().newXPath();
            HashMap<String, String> namespaces = new HashMap<String, String>();
            namespaces.put("xsi", "http://www.w3.org/2001/XMLSchema-instance");
            Node myRoot = nodePath.getOwnerDocument().getFirstChild();
            String myNamespace = myRoot.getNamespaceURI();
            namespaces.put("core", myNamespace);
            //ex: namespaces.put("core", "org.eclipse.jwt/1.3.0/core");
            xPath.setNamespaceContext(new CustomNamespaceContext(namespaces));

            XPathExpression expression = xPath.compile(xPathExpressionName);
            
            String document = createStringFromNode(myRoot);
            
            value = expression.evaluate(new InputSource(new StringReader(document)));
            
            // pb : les nodes "processes:Action" ont un name, mais pas "processes:InitialNode" et "processes:FinalNode"
            
            // début : pas d'activity dans XPDL
            // fin : value Final et activity BonitaEnd
            
            if (value.length() == 0) {
                // there is no name : check type
                XPathExpression expressionType = xPath.compile(xPathExpressionType);
                String valueType = expressionType.evaluate(new InputSource(new StringReader(document)));
                if (valueType.equals("processes:FinalNode")) {
                    value = "BonitaEnd";
                }
            }
            
//            System.out.println(path + ":" + value);
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        return value;
    }
    
    private static String applyPatternVar(String condition) {
        Pattern pattern = patternVar;
        Matcher matcher = pattern.matcher(condition);
        while (matcher.matches()) {
            condition = matcher.group(1) + matcher.group(2) + matcher.group(3);
            matcher = pattern.matcher(condition);
        }
        return condition;
    }
    
    private static String applyPatternsOp(String condition) {
        condition = applyPatternEqual(condition);
        condition = applyPatternDifferent(condition);
        return condition;
    }
    
    private static String applyPatternEqual(String condition) {
        StringBuilder result = new StringBuilder();
        
        Pattern pattern = patternEqual;
        Matcher matcher = pattern.matcher(condition);
        if (matcher.matches()) {
//            for (int i = 1; i <= matcher.groupCount(); i++) {
//                System.out.println(i + ":" + matcher.group(i));
//            }
            
            result.append(matcher.group(1));
            result.append(".equals(\"");
            result.append(matcher.group(2));
            result.append("\")");
            result.append(matcher.group(3));
            // traitement de la suite
            result.append(applyPatternsOp(matcher.group(4)));
        }
        else {
            result.append(condition);
        }
        return result.toString();
    }
    
    private static String applyPatternDifferent(String condition) {
        StringBuilder result = new StringBuilder();
        
        Pattern pattern = patternDifferent;
        Matcher matcher = pattern.matcher(condition);
        if (matcher.matches()) {
            result.append("!");
            result.append(matcher.group(1));
            result.append(".equals(\"");
            result.append(matcher.group(2));
            result.append("\")");
            result.append(matcher.group(3));
            // appel récursif pour la suite
            result.append(applyPatternsOp(matcher.group(4)));
        }
        else {
            result.append(condition);
        }
        
        return result.toString();
    }
    
    /**
     * in:  (Branche.Branche == tests &amp;&amp; isApproved.isApproved != true) || Reisemittel.Reisemittel == ok
     * out: (Branche.equals("tests") &amp;&amp; !isApproved.equals("true")) || Reisemittel.equals("ok")
     * 
     * guard:
     * <guard name="Approve" textualdescription="Approve" shortdescription="isApproved.isApproved == true">
     *     <detailedSpecification data="//@subpackages.2/@subpackages.1/@elements.4" attribute="isApproved" value="true"/>
     * </guard>
     *   
     * @param jwtCondition
     * @param guard
     * @param root
     * @return
     */
    public static String getCondition(final String jwtCondition, Node guard, Node root) {
        System.out.println("jwtCondition:" + jwtCondition);
        String condition = jwtCondition;
        
        condition = condition.replaceAll("&amp;", "&");
        condition = applyPatternVar(condition);
        condition = applyPatternsOp(condition);
        
        return condition;
        
//        pattern = patternDifferent;
//        matcher = pattern.matcher(condition);
//        while (matcher.matches()) {
//            result.append("!");
//            result.append(matcher.group(1));
//            result.append(".equals(\"");
//            result.append(matcher.group(2));
//            result.append("\")");
//            result.append(matcher.group(3));
//            conditionTemp = matcher.group(4);
//            matcher = pattern.matcher(conditionTemp);
//            if (!matcher.matches()) {
//                result.append(conditionTemp);                
//            }
//        }

//        conditionTemp = "";
//        pattern = patternDifferent;
//        matcher = pattern.matcher(condition);
//        while (matcher.matches()) {
//            result.append("!");
//            result.append(matcher.group(1));
//            result.append(".equals(\"");
//            result.append(matcher.group(2));
//            result.append("\")");
//            conditionTemp = matcher.group(3);
////            condition = matcher.group(1) + ".equals(\"" + matcher.group(2) + "\")" + matcher.group(3);
//            matcher = pattern.matcher(conditionTemp);
//        }
//        result.append(conditionTemp);

//        pattern = patternDifferent;
//        matcher = pattern.matcher(condition);
//        while (matcher.matches()) {
//            condition = "!" + matcher.group(1) + ".equals(\"" + matcher.group(2) + "\")" + matcher.group(3);
//            matcher = pattern.matcher(condition);
//        }
//        
//        return result.toString();
    }

    /**
     * Create a String result from DOM NodeList
     *
     * @param nodes
     *            the DOM Nodes. Must not be null
     * @return a String representation of the DOM Document
     * @throws TransformerException
     */
    public static String createStringFromNode(Node node) throws TransformerException {
        final StringWriter out = new StringWriter();
        final Result resultStream = new StreamResult(out);
        final TransformerFactory tFactory = TransformerFactory.newInstance();
        Transformer transformer;
        transformer = tFactory.newTransformer();
        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
        
//        node.normalize();
        final Source source = new DOMSource(node);
        transformer.transform(source, resultStream);
        
        return out.toString();
    }

    /**
     * Create a String result from DOM NodeList
     *
     * @param nodes
     *            the DOM Nodes. Must not be null
     * @return a String representation of the DOM Document
     * @throws TransformerException
     */
    public static String createStringFromNodeList(NodeList nodes) throws TransformerException {
        final StringWriter out = new StringWriter();
        final Result resultStream = new StreamResult(out);
        final TransformerFactory tFactory = TransformerFactory.newInstance();
        Transformer transformer;
        transformer = tFactory.newTransformer();
        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
        
        for (int i = 0; i < nodes.getLength(); i++) {
            Node node = nodes.item(i);
//            node.normalize();
            final Source source = new DOMSource(node);
            transformer.transform(source, resultStream);
        }
        
        return out.toString();
    }

}
