package org.eclipse.jwt.transformations.xpdl.impl;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.eclipse.jwt.transformations.base.TransformationException;
import org.eclipse.jwt.transformations.base.TransformationService;




public class Jwt2XpdlService extends TransformationService {
	
	private static String name = "XPDL (Simulator)";
	
	private static Jwt2XpdlService instance = null;
	
	public static Jwt2XpdlService getInstance() {
		if (instance == null) {
			synchronized (Jwt2XpdlService.class) {
				instance = new Jwt2XpdlService();
			}
		}
		return instance;
	}
	
	public Jwt2XpdlService() {
		
	}
	
	private String xsltResourcePath   = "org/eclipse/jwt/transformations/xpdl/impl/jwt2xpdl.xsl";

	
	/**
	 * TODO maybe other methods, ex. with EMF model or eclipse resource parameter ?
	 * @param inFilePath
	 * @param outFilePath
	 * @throws IOException
	 * @throws TransformationException
	 */
	public void transform(String inFilePath, String outFilePath) throws IOException, TransformationException {
        //String xmlIn  = "sample/approval/transformation_sample_3_approval_frompp.agilpro";
        //String xsltResourcePath   = "src/main/resources/jwt2xpdl.xsl";
        //String xmlOut = "target/out.xpdl";
        
        InputStream xsltIs = this.getClass().getClassLoader().getResourceAsStream(xsltResourcePath);
        if (xsltIs == null) {
        	throw new IOException("Can't find XSLT file in classpath " + xsltResourcePath);
        }
        
        File inFile = new File(inFilePath);
        if (!inFile.exists() || inFile.isDirectory() || !inFile.canRead()) {
        	throw new IOException("Can't open in file " + inFile.getAbsolutePath());
        }
        
        File outFile = new File(outFilePath);
        if (outFile.isDirectory() || !outFile.canWrite()) {
        	throw new IOException("Can't open out file " + outFile.getAbsolutePath());
        }

        Transformer transformer = null;
		try {
			TransformerFactory tFactory = TransformerFactory.newInstance();
			transformer = tFactory.newTransformer(new StreamSource(xsltIs));
		} catch (TransformerConfigurationException e) {
			throw new TransformationException("Can't init transformer from XSLT in classpath " + xsltResourcePath, e);
		} catch (TransformerFactoryConfigurationError e) {
			throw new TransformationException("Can't init transformer from XSLT in classpath " + xsltResourcePath, e);
		}

        try {
			transformer.transform(new StreamSource(inFile), new StreamResult(outFile));
		} catch (TransformerException e) {
			throw new TransformationException("Error transforming " + inFile.getAbsolutePath()
					+ " to " + outFile.getAbsolutePath() + " using " + xsltResourcePath, e);
		}
	}

	public static String getName() {
		return name;
	}

	public static void setName(String name) {
		Jwt2XpdlService.name = name;
	}
	
}
